;PatternEditor.s


peNavTrackLeft
	lda #00
	sta peHighlightActive
	lda PatternCursorX
	sbc #3
.(
	bcc skip1
	sta PatternCursorX
skip1	rts
.)

peNavTrackRight
	lda #00
	sta peHighlightActive

	lda PatternCursorX
	adc #4
	cmp #32
.(
	bcs skip1
	sta PatternCursorX
skip1	rts
.)

peNavLeft
	lda #00
	sta peHighlightActive
	lda PatternCursorX
	sec
	sbc #1
	and #31
	sta PatternCursorX
	rts
peNavRight
	lda #00
	sta peHighlightActive
	lda PatternCursorX
	clc
	adc #1
	and #31
	sta PatternCursorX
	rts

peNavUp
	lda #00
	sta peHighlightActive
	lda PatternCursorRow
	sec
	sbc #1
	and #63
	sta PatternCursorRow
	rts
	
peNavDown
	lda #00
	sta peHighlightActive
	lda PatternCursorRow
	clc
	adc #1
	and #63
	sta PatternCursorRow
	rts
	
peNavHome
	lda #00
	sta peHighlightActive
	lda #00
	sta PatternCursorRow
	rts
peNavEnd
	lda #00
	sta peHighlightActive
	lda #63
	sta PatternCursorRow
	rts

peToggleBlue
	lda peBlueRow
	eor #128
	sta peBlueRow
	rts
	

	

 
	

peOctave
;	sei
;	jmp peOctave
	;Parsed in A is the Note in the form of letters A thru G
	ldx PatternEntryType
	;If entry is Track Rest then abort
.(
	beq Failed
	;If Entry is Command then abort
	cpx #5
	beq Failed
	;If Entry is Rest, VRST or Bar then set default entry
	cpx #2
	bcs DefaultEntry

	;Reduce Keys 48 thru 57 to values 0-9
rent1	sec
	sbc #48
	tay
	
	;Perform comparison between entered octave and current octave
	lda PatternEntryLongNote	;This is independant of list offset
	
	cpy PatternEntryOctave	;This includes list offset
	beq Success
	bcc sub
	
	;Move current octave up to Selected octave
	sty loop1+1
	ldy PatternEntryOctave
loop1	cpy #00	;This includes list offset
	beq Success
	iny
	clc
	adc #12
	cmp #61
	bcc loop1
	rts	;Failed
	
sub	;Move current octave down to selected octave
	sty loop2+1
	ldy PatternEntryOctave
loop2	cpy #00	;This includes list offset
	beq Success
	
	dey
	sec
	sbc #12
	bcs loop2
Failed	rts	;Failed

DefaultEntry
	;We must permit specific Octave selection on rest entry
	;1) Backup selected octave
	pha
	;2) Write intermidiate Note value of C-0
	lda #00
	sta PatternEntrySFX
	sta PatternEntryLongNote
	lda #15
	sta PatternEntryVolume
	jsr peStorePatternLongNote
	;3) Fetch back
	jsr peFetchEntryData
	;4) Restore entered octave
	pla
	;5) Jump back to main
	jmp rent1

Success	sta PatternEntryLongNote
.)
	jmp peStorePatternLongNote


peNote	;The calculation of note is hindered by any Note Offset on the track
	;which directly effects the displayed note. So we must perform some
	;special code to ensure the desired Note is attainable in the current
	;octave otherwise abort.

	;Parsed in A is the Note in the form of letters A thru G
	ldx PatternEntryType
	;If entry is Track Rest then abort
.(
	beq Failed
	;If Entry is Command then abort
	cpx #5
	beq Failed
	;If Entry is Rest, VRST or Bar then set default entry
	cpx #2
	bcs DefaultEntry

rent1	;Convert Typed ABCDEFG(0-6) to Note 0-11(Ignore current Sharp)
	tay
	lda peActualNote-65,y
	sta EnteredNote
		
	;Take the Pattern entry up to B-X(or 60) by observing
	;calculated ShortNote
	ldx PatternEntryLongNote
	ldy PatternEntryNote
	cpy EnteredNote
	beq NoChange
loop1	cpy #11
	bcs BXreached
	iny
	inx
	cpx #61
	bcc loop1
BXreached
	;Then bring Pattern entry down to Typed Note. If Pattern
	;Entry <0 then Fail
loop2	cpy EnteredNote
	beq Success
	dey
	dex
	bpl loop2
	jmp Failed
DefaultEntry
	;We must permit specific Note selection on rest entry
	;1) Backup selected note
	pha
	;2) Write intermidiate Note value of C-0
	lda #00
	sta PatternEntrySFX
	sta PatternEntryLongNote
	lda #15
	sta PatternEntryVolume
	jsr peStorePatternLongNote
	;3) Fetch back
	jsr peFetchEntryData
	;4) Restore entered octave
	pla
	;5) Jump back to main
	jmp rent1
Success	stx PatternEntryLongNote
	jmp peStorePatternLongNote
Failed
NoChange	rts
.)

;	ldx #7
;
;loop1	cmp peKeyNote,x
;	beq skip3
;	dex
;	bpl loop1
;	rts
;skip3	lda peActualNote,x
;
;	sta PatternEntryNote
;	jmp peStorePatternNote
;skip1	;However if a rest or bar or VRest default to n-3 F0
;	cpx #5
;	beq skip2
;	ldx #03
;	stx PatternEntryOctave
;	ldx #00
;	stx PatternEntrySharp
;	;Also default Effect and Volume
;	stx PatternEntrySFX
;	ldx #15
;	stx PatternEntryVolume
;	sec
;	jmp rent1
;skip2	rts
;.)
	
;The Pattern Entry is composed of 
; Note/Rest/VRest/Bar/Command, Octave, Sharp, Volume and SFX, all within two bytes.
;It is extracted here from the two bytes into..
; PatternEntryType
;   0 Entry Inactive (Pattern Inactive)
;   1 Entry is a Note
;   2 Entry is a Rest
;   3 Entry is a VRest
;   4 Entry is a Bar
;   5 Entry is a Command
;					Store Back with..
; PatternEntryByte0				peStorePatternData
; PatternEntryByte1
;
; PatternEntryNote(Which includes Bar and Rest)	peStorePatternNote
; PatternEntrySharp
; PatternEntryOctave
;
; PatternEntryVolume			peStorePatternLongNote
; PatternEntrySFX
; PatternEntryLongNote
;
;These only apply to Track H if command has been applied
; PatternEntryCommand			peStorePatternCommand
; PatternEntryParam1
; PatternEntryParam2
;
peFetchEntryData
	;Calculate Address of Pattern Entry
	lda PatternCursorX
	lsr
	lsr
	tax
	;Branch if Track inactive
	lda TrackProperty,x
.(
	bpl PatternInactive

	lda PatternCursorRow
	jsr CalcPatternAddress
	
	; Extract bytes 0 & 1
	ldy #00
	lda (source),y
	sta PatternEntryLongNote
	sta PatternEntryByte0
	iny
	lda (source),y
	sta PatternEntrySFX
	sta PatternEntryByte1
	
	
	;Branch if Track H is command
	lda TrackProperty,x
	asl
	bmi PatternCommandEntry

	; Split into Note, SFX and Volume
	lda #00
	sta PatternEntryVolume
	lsr PatternEntryLongNote
	rol PatternEntryVolume
	lsr PatternEntryLongNote
	rol PatternEntryVolume
	lsr PatternEntrySFX
	rol PatternEntryVolume
	lsr PatternEntrySFX
	rol PatternEntryVolume
	
	; Branch if Rest, VRest or Bar
	lda PatternEntryLongNote
	cmp #61
	bcs BarRestVRest
	
	;Add List Note Offset
	adc tl_CurrentOctaveRange,x

	; Extract Note,Octave,Sharp
	ldx #255
	sec
loop1	inx
	sbc #12
	bcs loop1
	adc #12
	stx PatternEntryOctave
	tax
	lda Notes12ToNote,x
	sta PatternEntryNote
	lda Notes12ToSharp,x
	sta PatternEntrySharp
	
	;Set EntryType to 1
	lda #01
	sta PatternEntryType
	rts

PatternInactive
	;Set Entry Type to 0
	lda #0
	sta PatternEntryType
	rts

PatternCommandEntry
	;Extract Command
	lda PatternEntryLongNote
	and #15
	sta PatternEntryCommand
	
	; Extract Param1
	lda PatternEntryLongNote
	lsr
	lsr
	lsr
	lsr
	sta PatternEntryParam1
	
	; Extract Param2
	lda PatternEntrySFX
	sta PatternEntryParam2
	
	;Set Entry Type to 5
	lda #5
	sta PatternEntryType
	rts

BarRestVRest
	;Reduce 61-63 to 2-4 and store as EntryType
	sbc #59
	sta PatternEntryType
	
skip1	rts
.)

;Relies on (source) pointing to entry
peStorePatternData
	lda PatternEntryByte0
	ldy #00
	sta (source),y
	iny
	lda PatternEntryByte1
	sta (source),y
	rts
	
peStorePatternNote
	;Rebuild Long Note from Note,Sharp and Octave
	lda PatternEntryOctave
	asl
	asl
.(
	sta vector1+1
	asl
vector1	adc #00
.)
	;Add Note
	adc PatternEntryNote
	;Add Sharp
	adc PatternEntrySharp
	;Store as long note
	sta PatternEntryLongNote
peStorePatternLongNote
	;Based on..
	;A0   Volume B3
	;A1   Volume B2
	;A2-7 Note/Rst/VRST/Bar
	;B0   Volume B1
	;B1   Volume B0
	;B2-7 SFX
	lda PatternEntrySFX
	lsr PatternEntryVolume
	rol
	lsr PatternEntryVolume
	rol
	ldy #01
	sta (source),y
	lda PatternEntryLongNote
	lsr PatternEntryVolume
	rol
	lsr PatternEntryVolume
	rol
	dey
	sta (source),y
	rts

peStorePatternCommand
	lda PatternEntryParam1
	asl
	asl
	asl
	asl
	ora PatternEntryCommand
peStorePatternCommandBytes
	ldy #00
	sta (source),y
	iny
	lda PatternEntryParam2
	sta (source),y
	rts

	

peIncrement
	;Depends on (PatternCursorX AND 3) and PatternEntryType
	;If EntryType is 0(Inactive)
	;Ignore
	;If EntryType is 1(Note)
	;0 Increment Note by semitone
	;1 Increment Octave (0-4)
	;2 Increment Volume
	;3 Increment SFX
	;If EntryType is 2(Rest) or 4(Bar)
	;Ignore
	;If EntryType is 3(VRest)
	;0 Ignore
	;1 Ignore
	;2 Increment Volume
	;3 Ignore
	;If EntryType is 5(Command)
	;0 Ignore
	;1 Increment Command
	;2 Increment Param1
	;3 Depending on Command Increment Param2
	
	;Merge the two together to form a 23 Byte index to decide what to do
	lda PatternCursorX
	and #3
.(
	sta vector1+1
	lda PatternEntryType
	asl
	asl
vector1	ora #00
	tax
	lda IncrementWhatVectorHi,x
	beq skip1
	sta vector2+2
	lda IncrementWhatVectorLo,x
	sta vector2+1
	clc
vector2	jmp $dead
skip1	rts
.)

IncrementNote
	lda PatternEntryLongNote
	adc #1
	cmp #61
.(
	bcc skip1
	lda #00
skip1	sta PatternEntryLongNote
.)
	jmp peStorePatternLongNote

	
IncrementOctave
	lda PatternEntryOctave
	adc #1
	cmp #5
.(
	bcc skip1
	lda #00
skip1	sta PatternEntryOctave
.)
	jmp peStorePatternLongNote
	
IncrementVolume
	lda PatternEntryVolume
	adc #1
	cmp #16
.(
	bcc skip1
	lda #15
skip1	sta PatternEntryVolume
.)
	jmp peStorePatternLongNote

	
IncrementSFX
	lda PatternEntrySFX
	adc #1
	cmp #64
.(
	bcc skip1
	lda #63
skip1	sta PatternEntrySFX
.)
	jmp peStorePatternLongNote

IncrementCommand
	lda PatternEntryCommand
	adc #1
	cmp #16
.(
	bcc skip1
	lda #15
skip1	sta PatternEntryCommand
.)
	jmp peStorePatternCommand

IncrementParam1
	lda PatternEntryParam1
	adc #1
	cmp #16
.(
	bcc skip1
	lda #15
skip1	sta PatternEntryParam1
.)
	jmp peStorePatternCommand

IncrementParam2
	;Depends on Command
	ldx PatternEntryCommand
	lda ModifyParam2Flag,x
.(
	beq skip2
	inc PatternEntryParam2
	jmp peStorePatternCommand
skip2	rts
.)


IncrementWhatVectorLo
 .byt 0			;Inactive Channel
 .byt 0
 .byt 0
 .byt 0
 .byt <IncrementNote	;Note
 .byt <IncrementOctave
 .byt <IncrementVolume
 .byt <IncrementSFX
 .byt 0			;Rest
 .byt 0
 .byt 0
 .byt 0

 .byt 0			;VRest
 .byt 0
 .byt <IncrementVolume
 .byt 0

 .byt 0			;Bar
 .byt 0
 .byt 0
 .byt 0
 
 .byt 0			;Command
 .byt <IncrementCommand
 .byt <IncrementParam1
 .byt <IncrementParam2

IncrementWhatVectorHi
 .byt 0			;Inactive Channel
 .byt 0
 .byt 0
 .byt 0
 .byt >IncrementNote	;Note
 .byt >IncrementOctave
 .byt >IncrementVolume
 .byt >IncrementSFX
 .byt 0			;Rest
 .byt 0
 .byt 0
 .byt 0

 .byt 0			;VRest
 .byt 0
 .byt >IncrementVolume
 .byt 0

 .byt 0			;Bar
 .byt 0
 .byt 0
 .byt 0
 
 .byt 0			;Command
 .byt >IncrementCommand
 .byt >IncrementParam1
 .byt >IncrementParam2
;0  Trigger Out	Value to Write	ZP Location
;1  Trigger In	Value to detect	ZP Location
;2  Song Tempo	-		Tempo Frac
;3  SFX Tempo	-		Tempo Frac
;4  Pitchbend Up	Rate		Channel/s
;5  Pitchbend Down	Rate		Channel/s
;6  Spare
;7  Spare
;8  Spare
;9  Spare
;10 Rest		-		-
;11 Bar		-		-
ModifyParam2Flag
 .byt 1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0

;Pretty much opposite of increment	
peDecrement
	;Merge the two together to form a 23 Byte index to decide what to do
	lda PatternCursorX
	and #3
.(
	sta vector1+1
	lda PatternEntryType
	asl
	asl
vector1	ora #00
	tax
	lda DecrementWhatVectorHi,x
	beq skip1
	sta vector2+2
	lda DecrementWhatVectorLo,x
	sta vector2+1
	sec
vector2	jsr $dead
skip1	rts
.)

DecrementNote
	lda PatternEntryLongNote
	sbc #1
.(
	bcs skip1
	lda #00
skip1	sta PatternEntryLongNote
.)
	jmp peStorePatternLongNote

	
DecrementOctave
	lda PatternEntryOctave
	sbc #1
.(
	bcs skip1
	lda #00
skip1	sta PatternEntryOctave
.)
	jmp peStorePatternLongNote
	
DecrementVolume
	lda PatternEntryVolume
	sec
	sbc #1
.(
	bcs skip1
	lda #00
skip1	sta PatternEntryVolume
.)
	jmp peStorePatternLongNote
	
DecrementSFX
	lda PatternEntrySFX
	sec
	sbc #1
.(
	bcs skip1
	lda #00
skip1	sta PatternEntrySFX
.)
	jmp peStorePatternLongNote

DecrementCommand
	lda PatternEntryCommand
	sec
	sbc #1
.(
	bcs skip1
	lda #00
skip1	sta PatternEntryCommand
.)
	jmp peStorePatternCommand

DecrementParam1
	lda PatternEntryParam1
	sec
	sbc #1
.(
	bcs skip1
	lda #00
skip1	sta PatternEntryParam1
.)
	jmp peStorePatternCommand

DecrementParam2
	;Depends on Command
	ldx PatternEntryCommand
	lda ModifyParam2Flag,x
.(
	beq skip2
	dec PatternEntryParam2
	jmp peStorePatternCommand
skip2	rts
.)


DecrementWhatVectorLo
 .byt 0			;Inactive Channel
 .byt 0
 .byt 0
 .byt 0
 .byt <DecrementNote	;Note
 .byt <DecrementOctave
 .byt <DecrementVolume
 .byt <DecrementSFX
 .byt 0			;Rest
 .byt 0
 .byt 0
 .byt 0

 .byt 0			;VRest
 .byt 0
 .byt <DecrementVolume
 .byt 0

 .byt 0			;Bar
 .byt 0
 .byt 0
 .byt 0
 
 .byt 0			;Command
 .byt <DecrementCommand
 .byt <DecrementParam1
 .byt <DecrementParam2

DecrementWhatVectorHi
 .byt 0			;Inactive Channel
 .byt 0
 .byt 0
 .byt 0
 .byt >DecrementNote	;Note
 .byt >DecrementOctave
 .byt >DecrementVolume
 .byt >DecrementSFX
 .byt 0			;Rest
 .byt 0
 .byt 0
 .byt 0

 .byt 0			;VRest
 .byt 0
 .byt >DecrementVolume
 .byt 0

 .byt 0			;Bar
 .byt 0
 .byt 0
 .byt 0
 
 .byt 0			;Command
 .byt >DecrementCommand
 .byt >DecrementParam1
 .byt >DecrementParam2
	rts

	
peReset	;Are we over Left two characters of Track?
	lda PatternCursorX
	and #3
	
	cmp #2
.(
	bcs OverParameters
	; Is this a Command Track?
	lda #4
	ldx PatternEntryType
	cpx #5
	beq CommandRest
	;Is this a note entry?
	lda #244
	cpx #1
	beq CommandRest
	; Is this a Rest entry?
	lda #62*4
	cpx #2
	beq CommandRest
	; Is this a VRest entry?
	lda #61*4
	cpx #3
	bne skip1
CommandRest
	sta PatternEntryByte0
	lda #00
	sta PatternEntryByte1
	jmp peStorePatternData
OverParameters
	;Is this a command entry?
	ldx PatternEntryType
	cpx #5
	beq ResetCommandParams
	;Is this a note?
	cpx #1
	bne skip1
ResetNoteParams
	tax
	lda #00
	sta PatternEntryVolume-2,x
	jmp peStorePatternLongNote
ResetCommandParams
	tax
	lda #00
	sta PatternEntryParam1-2,x
	jmp peStorePatternCommand
skip1	rts
.)	


peCopyLast
	lda PatternCursorRow
.(
	beq skip1
	dec PatternCursorRow
	jsr peFetchEntryData
	inc PatternCursorRow
	lda PatternCursorX
	lsr
	lsr
	tax
	lda PatternCursorRow
	jsr CalcPatternAddress
	jmp peStorePatternData
skip1	rts
.)
	
peCopyNext
	lda PatternCursorRow
	cmp #63
.(
	bcs skip1
	inc PatternCursorRow
	jsr peFetchEntryData
	dec PatternCursorRow
	lda PatternCursorX
	lsr
	lsr
	tax
	lda PatternCursorRow
	jsr CalcPatternAddress
	jmp peStorePatternData
skip1	rts
.)

;Grab acts in different ways, depending on the entry type
peGrab
	lda PatternEntryByte0
	sta GrabbedPatternEntry0
	lda PatternEntryByte1
	sta GrabbedPatternEntry1
	jmp peDisplayStatus
	
peDrop
	lda GrabbedPatternEntry0
	sta PatternEntryByte0
	lda GrabbedPatternEntry1
peDropRent1
	sta PatternEntryByte1
	jmp peStorePatternData
	
peBar	;
	lda PatternEntryType
.(
	beq skip1
	lda #%11111100
	sta PatternEntryByte0
	lda #00
	jmp peDropRent1
skip1	rts
.)

peHighlightDown
	lda peHighlightActive
	beq peInitialiseHighlighting
	jsr peNavDown
	lda #01
	sta peHighlightActive
	lda PatternCursorRow
	sta peHighlightRowEnd
	rts

peCalcCurrentTrack
	lda PatternCursorX
	lsr
	lsr
	rts
	
peInitialiseHighlighting
	;Cannot begin highlighting in Rested Pattern
	lda PatternEntryType
.(
	beq skip1
	;Store Current Track and Row
	jsr peCalcCurrentTrack
	sta peHighlightTrackStart
	sta peHighlightTrackEnd
	lda PatternCursorRow
	sta peHighlightRowStart
	sta peHighlightRowEnd
	lda #01
	sta peHighlightActive
skip1	rts
.)

peDeactivateHighlighting
	lda #00
	sta peHighlightActive
	rts

peHighlightUp
	;Is Highlight active?
	lda peHighlightActive
.(
	beq skip1
	
	;Does move make Pattern Cursor above Highlight start row?
	jsr peNavUp
	lda #01
	sta peHighlightActive
	lda PatternCursorRow
	cmp peHighlightRowStart
	bcc peDeactivateHighlighting
	sta peHighlightRowEnd
skip1	rts
.)
	
peHighlightLeft
	lda peHighlightActive
.(
	beq skip1
	
	;Does move make Pattern Cursor left of Highlight start track?
	jsr peNavTrackLeft
	lda #01
	sta peHighlightActive
	jsr peCalcCurrentTrack
	cmp peHighlightTrackStart
	bcc peDeactivateHighlighting
	sta peHighlightTrackEnd
skip1	rts
.)

peHighlightRight
	lda peHighlightActive
	beq peInitialiseHighlighting
	jsr peNavTrackRight

	;Moving into a rest channel will deactivate highlight
	lda PatternEntryType
.(
	beq skip1
	lda #01
	sta peHighlightActive
	jsr peCalcCurrentTrack
	sta peHighlightTrackEnd
skip1	rts
.)

	
	

peCopy
	lda #00
peCopy2Buffer
	sta peTransferType
	lda peHighlightActive
.(
	beq skip1
	lda #00
	sta CopyByteCount
	sta CopyByteCount+1
	;Display copying..
	ldx #1
	ldy #27
	lda #0+DSM_INVERT
	jsr DisplaySimpleMessage
	;Setup for copying
	jsr SetupCopyBuffer
	;Transfer Boundaries to Copy otherwise highlighting will corrupt original
	lda peHighlightTrackStart
	sta peCopyTrackStart
	lda peHighlightTrackEnd
	sta peCopyTrackEnd
	lda peHighlightRowStart
	sta peCopyRowStart
	lda peHighlightRowEnd
	sta peCopyRowEnd
	;Copy area
	ldx peHighlightTrackStart

loop2	ldy peHighlightRowStart
loop1	jsr peFetchHilightedEntry
	jsr peStore2CopyBuffer
	iny
	cpy peHighlightRowEnd
	beq loop1
	bcc loop1
	inx
	cpx peHighlightTrackEnd
	beq loop2
	bcc loop2

	;Display Copied and clear previous
	ldx #1
	ldy #27
	lda #1+DSM_CLEARPREVIOUS+DSM_INVERT
	jsr DisplaySimpleMessage
	;Dehighlight area
	lda #00
	sta peHighlightActive
	;Set copy buffer to Pattern data
	lda CurrentEditor
	sta CopyEditor
	sec
skip1	rts
.)
peCut	lda #01
	jmp peCopy2Buffer

;Fetch Highlighted Bytes 0 and +1 using X(Track) and Y(Row) without corrupting either
peFetchHilightedEntry
	sty peTempY
	;Fetch Address
	tya
	jsr CalcPatternAddress
	ldy #00
	lda peTransferType
.(
	bne skip1
	lda (source),y
	sta cpyByte0
	iny
	lda (source),y
	sta cpyByte1
	ldy peTempY
	rts
skip1	;Cut
.)
	lda (source),y
	sta cpyByte0
	lda #61*4
	sta (source),y
	iny
	lda (source),y
	sta cpyByte1
	lda #0
	sta (source),y
	ldy peTempY
	rts
	
;Store the Bytes cpyByte0 and cpyByte1 to the CopyBuffer sequentially without corrupting X or Y
peStore2CopyBuffer
	sty peTempY
	ldy #00
	lda cpyByte0
	sta (copy),y
	iny
	lda cpyByte1
	sta (copy),y
	lda #2
	jsr Add_Copy
	lda #2
	jsr Add_CopyCount
	ldy peTempY
	rts
	

pePaste
	lda #00
	jmp peCopy2Pattern
peMerge
	lda #1
peCopy2Pattern
	sta peTransferType
	lda CopyByteCount
.(
	bne skip2
	lda CopyByteCount+1
	beq skip1
skip2	lda CopyEditor
	cmp CurrentEditor
	bne skip1
	
	;Setup for copying
	jsr SetupCopyBuffer

	;Use Copy boundaries purely to track original format
	lda PatternCursorRow
	sta pecCurrentRow
	jsr peCalcCurrentTrack
	sta pecCurrentTrack
	
	
	ldx peCopyTrackStart

loop2	ldy peCopyRowStart

	;Abort if attempting to paste into inactive Track
	lda TrackProperty,x
	bpl skip1

loop1	jsr peFetchCopyEntry
	jsr peStore2Pattern
	inc pecCurrentRow
	;Prevent pasting beyond entry 63
	lda pecCurrentRow
	cmp #64
	bcs skip1
	iny
	cpy peCopyRowEnd
	beq loop1
	bcc loop1
	lda PatternCursorRow
	sta pecCurrentRow
	inc pecCurrentTrack
	;Prevent pasting into Track8!
	lda pecCurrentTrack
	cmp #8
	bcs skip1
	inx
	cpx peCopyTrackEnd
	beq loop2
	bcc loop2
skip1	rts
.)
	
peFetchCopyEntry
	sty peTempY
	ldy #00
	lda (copy),y
	sta cpyByte0
	iny
	lda (copy),y
	sta cpyByte1
	lda #2
	jsr Add_Copy
	ldy peTempY
	rts
	
peStore2Pattern
	sty peTempY
	stx peTempX
	lda pecCurrentRow
	ldx pecCurrentTrack
	jsr CalcPatternAddress
	ldy #00
	lda peTransferType
.(
	bne Merge
skip1	lda cpyByte0
	sta (source),y
	lda cpyByte1
	iny
	sta (source),y
	ldx peTempX
	ldy peTempY
	rts
Merge
	;In Merge only paste if the entry is a Rest(61*4)
	lda (source),y
	lsr
	lsr
	cmp #61
	beq skip1
.)
	ldx peTempX
	ldy peTempY
	rts
	
pePlayPattern
	jmp PlayPattern
pePlaySong
	rts
peMuteTrack
	;Calc Current Track
	lda PatternCursorX
	lsr
	lsr
	
	;Use as index to bitpos
	tax
	lda Bitpos,x
	
	;And toggle mute flag
	eor prActiveTracks_NotMuted
	sta prActiveTracks_NotMuted
	
	;Play monitor should automatically pick this up
	rts



;This depends on 1 of 4 entries or Command Track H


peToggle
	rts
peInsert
	lda PatternCursorRow
	cmp #63
.(
	bcs skip2
	;Insert row(Rest) in all active tracks and shift data down
	lda PatternCursorX
	lsr
	lsr
	tax
	lda TrackProperty,x
	bpl skip2
	ldy #62	;Row
loop1	jsr peFetchPatternEntry
	iny
	jsr peStorePatternEntry
	dey
	dey
	bmi skip3
	cpy PatternCursorRow
	beq loop1
	bcs loop1
skip3	ldy PatternCursorRow
	jsr peStorePatternRest
skip2	rts
.)
peDelete
	lda PatternCursorRow
	cmp #63
.(
	bcs skip2
	;Delete row in all active tracks and shift data up(Storing rest in entry 63)
	lda PatternCursorX
	lsr
	lsr
	tax
	lda TrackProperty,x
	bpl skip2
	ldy PatternCursorRow
	iny
loop1	jsr peFetchPatternEntry
	dey
	jsr peStorePatternEntry
	iny
	iny
	cpy #64
	bcc loop1
	ldy #63
	jsr peStorePatternRest
skip2	rts
.)

;Parsed
;X Track
;Y Row
;Returns
;cpyByte0	Pattern Entry Byte0
;cpyByte1	Pattern Entry Byte1
peFetchPatternEntry
	tya
	pha
	jsr CalcPatternAddress
.(
	ldy #01
loop1	lda (source),y
	sta cpyByte0,y
	dey
	bpl loop1
.)
	pla
	tay
	rts
	
peStorePatternEntry
	tya
	pha
	jsr CalcPatternAddress
.(
	ldy #01
loop1	lda cpyByte0,y
	sta (source),y
	dey
	bpl loop1
.)
	pla
	tay
	rts
	
peStorePatternRest
	lda #10
	cpx #7
.(
	bcs skip1
	lda #61*4
skip1	sta cpyByte0
.)
	lda #00
	sta cpyByte1
	jmp peStorePatternEntry
	
peMoveEntryDown
	;Must ensure boundaries are not exceeded
	lda PatternCursorRow
.(
	beq skip1
	cmp #63
	bcs skip1
	inc PatternCursorRow
	jsr peDelete
	dec PatternCursorRow
;	dec PatternCursorRow
	jsr peInsert
;	inc PatternCursorRow
	inc PatternCursorRow
skip1	rts
.)
	

	
	
	
peMoveEntryUp
	;Just do reverse of MoveDown
	lda PatternCursorRow
.(
	beq skip1
	cmp #63
	bcs skip1
	dec PatternCursorRow
	jsr peDelete
	inc PatternCursorRow
;	inc PatternCursorRow
	jsr peInsert
	dec PatternCursorRow
;	dec PatternCursorRow
skip1	rts
.)
	rts

peToggleCommand
	;Is Track H Command?
	lda TrackProperty+7
	asl
.(
	bpl skip1
	; Fetch Command
	ldx #7
	ldy PatternCursorRow
	jsr peFetchPatternEntry
	lda cpyByte0
	and #15
	; Is Command Track specific? (3 Only)
	cmp #3
	bne skip1
	; invert Current Track bitpos in cpyByte1
	lda PatternCursorX
	lsr
	lsr
	tax
	lda cpyByte1
	eor Bitpos,x
	sta cpyByte1
	ldx #7
	ldy PatternCursorRow
	jsr peStorePatternEntry
skip1	rts
.)

peCmdTriggerOut
	;Is Track H Command?
	lda TrackProperty+7
	asl
.(
	bpl skip1
	
	; Fetch H Entry
	ldx #7
	ldy PatternCursorRow
	jsr peFetchPatternEntry
	
	;A0-3 Command ID(0-11)
	;A4-7 Param1(0-15)
	;B0-7 Param2(0-255)
	
	;Store Command and Default Param1
	lda #0

	sta cpyByte0
	sta cpyByte1
	
	ldx #7
	ldy PatternCursorRow
	jsr peStorePatternEntry
skip1	rts
.)
	
	
peCmdTriggerIn
	;Is Track H Command?
	lda TrackProperty+7
	asl
.(
	bpl skip1
	
	; Fetch H Entry
	ldx #7
	ldy PatternCursorRow
	jsr peFetchPatternEntry
	
	;A0-3 Command ID(0-11)
	;A4-7 Param1(0-15)
	;B0-7 Param2(0-255)
	
	;Store Command and Default Param1
	lda #1

	sta cpyByte0
	lda #00
	sta cpyByte1
	
	ldx #7
	ldy PatternCursorRow
	jsr peStorePatternEntry
skip1	rts
.)
peCmdSongTempo
	;Is Track H Command?
	lda TrackProperty+7
	asl
.(
	bpl skip1
	
	; Fetch H Entry
	ldx #7
	ldy PatternCursorRow
	jsr peFetchPatternEntry
	
	;A0-3 Command ID(0-11)
	;A4-7 Param1(0-15)
	;B0-7 Param2(0-255)
	
	;Store Command and Default Param1
	lda #2

	sta cpyByte0
	lda #10	;Default tempo to 10
	sta cpyByte1
	
	ldx #7
	ldy PatternCursorRow
	jsr peStorePatternEntry
skip1	rts
.)

peCmdBend
	;Is Track H Command?
	lda TrackProperty+7
	asl
.(
	bpl skip1
	
	; Fetch H Entry
	ldx #7
	ldy PatternCursorRow
	jsr peFetchPatternEntry
	
	;A0-3 Command ID(0-11)
	;A4-7 Param1(0-15)
	;B0-7 Param2(0-255)
	
	;Store Command and Default Param1
	lda #3

	sta cpyByte0
	lda #10	;Default param2 to 10
	sta cpyByte1
	
	ldx #7
	ldy PatternCursorRow
	jsr peStorePatternEntry
skip1	rts
.)

