;Track Music
;For use in jumping between editors and ensuring editors get correct music data to work with
;Consolidate routines..
;1) SynchroniseMusicPointers
;2) TrackList
;3) TrackCurrentSongDetails

;Playing Song from List		- Locates Song Start
;Playing Track row from List		- Tracks up to Track row(Capture List Song Tempo separately!)
;Playing Pattern from Patterns	- Tracks up to Track row(Capture List Song Tempo separately!)
;Playing Song from Patterns		- Locates Song Start
;Playing SFX			- Tracks SFX IRQ and Pattern Note/Volume/SFX
;Jump Pattern from list		- Tracks settings from song start
;Jump SFX from list			- Tracks up to Pattern entry Note
;Jump SFX from Patterns		- Tracks up to Pattern entry Note
;Jump Patterns from SFX		- Tracks Pattern config from list
;Should return carry if jump permitted(All links Valid)

TrackMusic
	;Track List from song start to Current Edit line
	jsr tmSetDefaults
	jsr FetchSongStartListRow
	sta tmListRow
.(	
loop1	;Fetch List row data
	ldy tmListRow
	jsr eeFetchRowData
	
	;Is RWC?
	lda eeRowData+1
	cmp #%11000000
	bcc skip2
	lda eeRowData
	and #7
	cmp #1
	beq EndSongFound
	jsr tmProcessRWC
	jmp skip1
	
skip2	;>Compile Track Patterns
	lda #00
	sta tmTrackIndex
	sta tmListTrackOffset
	
loop2	;Exclude Mimicking
	ldx tmListTrackOffset
	ldy tmTrackIndex
	lda eeRowData+1,x
	cmp #%01000000
	bcs Mimicking
	
	;Extract SS
	and #63
	sta tmCurrentSS,y
	
	;Exclude Track Rest
	lda eeRowData,x
	and #127
	cmp #127
	bcs skip4
	
	;Reset CommandH Flag
	ldx #00
	sta tmCommandHFlag
	
	;Scan pattern if Current-ListRow
	sta tmCurrentPattern,y
	ldx tmListRow
	cpx ListCursorRow
	beq Scan
	
	;Scan pattern if CommandH
	cpy #7
	bne skip4
	lda tmCurrentSS,y
	cmp #63
	bcc skip4
	;Set CommandH Flag
	sta tmCommandHFlag
	
Scan	tya
	tax
	jsr tmScanPattern
	
	;Current ListRow?
	ldx tmListRow
	cpx ListCursorRow
	beq Finish
	
Mimicking
skip4	inc tmListTrackOffset
	inc tmListTrackOffset
	inc tmTrackIndex
	lda tmTrackIndex
	cmp #8
	bcc loop2
	
skip3	inc tmListRow
	bpl loop1
Finish
.)
	rts

;If a Song Start is found the tempo is stored twice. once for updating in pattern
;and once for playing pattern.
;0 New Music 	- Update List Tempo, Update Global Tempo
;1 End of Song	- ?
;2 Fade Music 	- Ignore
;3 IRQ Rates	- Update SFX IRQ, Update Music IRQ
;4 Sharing Behaviour- Update Sharing Ticks, Update Behaviour rule
;5 Note Offset	- Update Note Offsets
;6 Resolutions	- Update Noise Res, Update Volume Res

tmProcessRWC
	tax
	
	

;Scan up to pattern cursor (if list row is cursor row) or end of pattern
;Scan track H commands
tmScanPattern
	ldy #00
	sty tmPatternIndex
.(
loop1	;Calc Address of current line in Pattern
	lda tmPatternIndex
	asl
	sta vector1+1
	lda tmCurrentPattern,x	;0-127)
	lsr
	pha
	lda #00
	ror
	;Add Row Offset
vector1	adc #00
	sta source
	pla
	adc #>PatternMemory
	sta source+1
	
	;Branch if CommandH
	ldy #00
	lda tmCommandHFlag
	beq tmScanPatternNoteEntry
tmScanCommandHEntry
	;Extract Command
	lda (source),y
	and #15
	
	;We are interested only if Note Tempo Change
	cmp #2
	bne skip1
	
	;Capture Note Tempo Change
	iny
	lda (source),y
	sta tmNoteTempo
skip1	jmp skip2

tmScanPatternNoteEntry
	;Extract LongNote,Volume and SFX 
	sty tmTrackVolume,x
	lda (source),y
	lsr
	rol tmTrackVolume,x
	lsr
	rol tmTrackVolume,x
	sta tmTrackLongNote,x
	iny
	lda (source),y
	lsr
	rol tmTrackVolume,x
	lsr
	rol tmTrackVolume,x
	sta tmTrackSFX,x
	
skip2	;Terminate if index on current cursor row
	lda tmPatternIndex
	cmp PatternCursorRow
	beq skip3
	inc tmPatternIndex
	bpl loop1
skip3	rts
.)	
	


