;prProcessPatternCommand.s

;Command Track H
;A0-3 Command ID(0-15)
;  0     TriggerOut
;  1     TriggerIn 
;  2     NoteTempo 
;  3     PitchBend 
;  4-11  Rest
;  12-15 Bar
;A4-7 Parameter1 B0-3(0-15)
;B0-7 Parameter2 B0-7(0-255)
ProcessCommandTrackH
	; Capture Param2
	iny
	lda (pattern),y
	sta prTemp01
	; Detect bar in next entry
	iny
	lda (pattern),y
	and #15
	cmp #12
.(
	bcc skip1
	sta prNextEntryIsBarFlag

skip1	; Capture Param1
.)
	dey
	dey
	lda (pattern),y
	lsr
	lsr
	lsr
	lsr
	sta prTemp02
	; Capture Command ID
	lda (pattern),y
	and #15
	cmp #4
.(
	bcs CommandRest
	tay
	lda prPatternCommandVectorLo,y
	sta vector1+1
	lda prPatternCommandVectorHi,y
	sta vector1+2
	;Backup X
	stx prPatternCommandX
vector1	jsr $dead
	ldx prPatternCommandX
CommandRest
.)
	rts
	
prpc_TriggerOut
prpc_TriggerIn
	;Since we are in the editor environment it is not neccesary to process either trigger
	;command
	rts
prpc_NoteTempo
	;Use Parameter1 for Note Tempo (Show as c2-?)
	lda prTemp01
	sta prNoteDelayRefer
	rts
prpc_PitchBend
;	nop
;	jmp prpc_PitchBend
	;Pitchbend must work as follows..
	;1)Split Rate(0-15) into Delay(B0-1) and Step(B2-3)
	ldx prTemp02
	lda pbDelayTable,x
	sta pbTemp03	;Delay
	lda pbStepTable,x
	sta pbTemp02	;Step
	;Backup prTemp01 because conversion of pitch will corrupt it
	lda prTemp01
	sta pbTemp01
	;2)For each Track in Param2(prTemp01)..
	; 2.1)locate next note in pattern
	;    2.1.1)convert to pitch
	;    2.1.2)store as destination pitch
	; 2.2)Read Param1 as Pitchbend Rate
	; 2.3)Decide on direction so that check can be performed after change
	;     and indefinate oscillation around destination never happens.
	;prProcPitchbend will raise or lower pitch to reach destination at rate
	;                then stop once reached (disabling Pitchbend on Track)
	ldx #6
.(	
loop2	lda pbTemp01
	and Bitpos,x
	and prActiveTracks_Pattern
	beq skip1
	;Rapidly calc Pattern Address
	lda prTrackPattern,x
	lsr
	sta prTemp01
	lda #00
	ror
	sta pbPattern
	lda prTemp01
	adc #>PatternMemory
	sta pbPattern+1
	;Fetch next Pattern Entry
	ldy prPatternIndex
loop1	iny
	iny
	;Abort on end of pattern
	bmi skip1
	lda (pbPattern),y
	;Abort on Bar
	cmp #63*4
	bcs skip1
	;Loop if not Note
	cmp #61*4
	bcs loop1
	;Shift to extract next note for track
	lsr
	lsr
	;Add Note Base for track
	clc
	adc prBaseOctave,x
	;convert note to Pitch
	jsr ConvertToPitch	;Corrupts prTemp01/02
	;And store
	sta PitchbendDestinationLo,x
	sty PitchbendDestinationHi,x
	;Calc direction
	ldy #1	;Default to increment
	lda prTrackPitchHi,x
	cmp PitchbendDestinationHi,x
	bcc increment
	bne decrement
	lda prTrackPitchLo,x 
	cmp PitchbendDestinationLo,x
	beq skip1	;Matched - Abort
	bcc increment
decrement	ldy #255
increment	sty PitchbendDirection,x
	;Store Pitchbend Delay
	lda pbTemp03
	sta PitchbendDelay,x
	sta PitchbendDelayCount,x
	;Store Pitchbend Step
	lda pbTemp02
	sta PitchbendStep,x
	;Activate Pitchbend on Track
	lda Bitpos,x
	ora prActiveTracks_Pitchbend
	sta prActiveTracks_Pitchbend
	;Proceed to next Track
skip1	dex
	bpl loop2
.)
	rts
                      	
	
prProcPitchbend
	ldx #7
.(
loop1	lda Bitpos,x
	and prActiveTracks_Pitchbend
	beq skip1
	; Delay this pitchbend
	dec PitchbendDelayCount,x
	bpl skip1
	lda PitchbendDelay,x
	sta PitchbendDelayCount,x
	; Decide which direction to go
	lda prTrackPitchLo,x
	ldy PitchbendDirection,x
	bpl increment
decrement	sec
	sbc PitchbendStep,x
	tay
	lda prTrackPitchHi,x
	sbc #00
	;Now perform comparison
	cmp PitchbendDestinationHi,x
	bcc Reached
	sta prTrackPitchHi,x
	beq skip4
	tya
	jmp skip3
skip4	tya
	cmp PitchbendDestinationLo,x
	beq Matched	;Matched - Abort
	bcc Reached
skip3	sta prTrackPitchLo,x
	jmp skip1
Reached	;With reached set tracks pitch to destination pitch
	lda PitchbendDestinationLo,x
	sta prTrackPitchLo,x
	lda PitchbendDestinationHi,x
	sta prTrackPitchHi,x
Matched	;Disable Pitchbend
	lda BitposMask,x
	and prActiveTracks_Pitchbend
	sta prActiveTracks_Pitchbend
	jmp skip1
increment	clc
	adc PitchbendStep,x
	tay
	lda prTrackPitchHi,x
	adc #00
	;Now perform comparison
	sta prTrackPitchHi,x
	cmp PitchbendDestinationHi,x
	beq skip2
	bcs Reached
	tya
	jmp skip5
skip2	tya
	cmp PitchbendDestinationLo,x
	bcs Reached
skip5	sta prTrackPitchLo,x
skip1	dex
	bpl loop1
.) 
	rts
